## An Introduction to Biostatistical Analysis with R
## R codes for lecture on 04/14/2017

## 2rd step: Find relationship between variables using regression

# this data set was analyzed in Zhao 2011 (Nature Communications 2:467)
pheno <- read.csv("RiceDiversityPheno.csv")
line <- read.csv("RiceDiversityLine.csv")
line.pheno <- merge(line, pheno, by.x = "NSFTV.ID", by.y = "NSFTVID")
head(line.pheno)

# extract variables for regression analysis
data <- data.frame(
	height = line.pheno$Plant.height,
	flower = line.pheno$Flowering.time.at.Arkansas,
	PC1 = line.pheno$PC1,
	PC2 = line.pheno$PC2,
	PC3 = line.pheno$PC3,
	PC4 = line.pheno$PC4)
data <- na.omit(data)

# look at the relationship between plant height and flowering time
plot(data$height ~ data$flower)

# perform single linear regression
model <- lm(height ~ flower, data = data)
summary(model)

# again, plot the two variables
plot(data$height ~ data$flower)
abline(model, col = "red")

# calculate fitted values
height.fit <- fitted(model)
points(data$flower, height.fit, pch = 3, col = "green")

# plot residuals
segments(data$flower, height.fit,
		 data$flower, height.fit + resid(model), col = "gray")
# height = height.fit + resid(model)

# predict unknown data
height.pred <- predict(model, data.frame(flower = seq(60, 140, 20)))
points(seq(60, 140, 20), height.pred, pch = 2, col = "blue")

######
#calculate by ourselves
x <- data$flower
y <- data$height
require(rgl)
mu <- seq(0, 100, 1)
beta <- seq(0, 2, 0.02)
sse <- matrix(NA, length(mu), length(beta))
for(i in 1:length(mu)) {
	for(j in 1:length(beta)) {
		sse[i, j] <- sum((y - mu[i] - beta[j] * x)^2)
	}
}
persp3d(mu, beta, sse, col = "green")

n <- length(x)
ssx <- sum(x^2) - n * mean(x)^2
ssxy <- sum(x * y) - n * mean(x) * mean(y)
b <- ssxy / ssx
b
m <- mean(y) - b * mean(x)
m

# draw scatter plot and regression line 
plot(y ~ x)
abline(m, b)

# calculate fitted values
y.hat <- m + b * x
lim <- range(c(y, y.hat))
plot(y, y.hat, xlab = "Observed", ylab = "Fitted", xlim = lim, ylim = lim)
abline(0, 1)

# calculate correlation between observed and fitted values
cor(y, y.hat)

# compare the square of the correlation and R2 
cor(y, y.hat)^2
summary(model)

###### 
# analysis of variance of regression
model <- lm(height ~ flower, data = data)
anova(model)

### ANOVA--ANOVA--ANOVA...
ssr <- b * ssxy
ssr
ssy <- sum(y^2) - n * mean(y)^2
sse <- ssy - ssr
sse

msr <- ssr / 1
msr
mse <- sse / (n - 2)
mse

f.value <- msr / mse
f.value

1 - pf(f.value, 1, n - 2)

# check the summary of the result of regression analysis		 
summary(model)

sqrt(mse)

ssr / ssy

(ssy / (n - 1) - mse) / (ssy / (n - 1))

# confidnence interval of parameters

# test beta = 0
t.value <- (b - 0) / sqrt(mse/ssx)
t.value
2 * (1 - pt(t.value, n - 2))

summary(model)

# test beta = 0.5
t.value <- (b - 0.5) / sqrt(mse/ssx)
t.value
2 * (1 - pt(t.value, n - 2))

# test mu = 0
t.value <- (m  - 0) / sqrt(mse * (1/n + mean(x)^2 / ssx))
t.value
2 * (1 - pt(t.value, n - 2))

# test mu = 60
t.value <- (m  - 50) / sqrt(mse * (1/n + mean(x)^2 / ssx))
t.value
2 * (1 - pt(t.value, n - 2))


summary(model)

# predict values
pred <- predict(model)
head(pred)
head(fitted(model))
# this is just fitted values

# calculate confidence interval
pred <- predict(model, interval = "confidence", level = 0.95)
head(pred)

# draw confidence and prediction bands
pred <- data.frame(flower = 50:160)
pc <- predict(model, int = "c", newdata = pred)
plot(data$height ~ data$flower)
matlines(pred$flower, pc, lty = c(1, 2, 2), col = "red")

# calculate confidence interval
x <- 50:160
tv <- qt(0.975, n - 2)
y.hat <- m + b * x
mean.x <- mean(data$flower)
y.hat.upper <- y.hat + tv * sqrt(mse) * sqrt(1/n + (x - mean.x)^2 / ssx)
y.hat.lower <- y.hat - tv * sqrt(mse) * sqrt(1/n + (x - mean.x)^2 / ssx)
plot(data$height ~ data$flower)
matlines(x, cbind(y.hat, y.hat.upper, y.hat.lower), lty = c(1, 2, 2), col = "red")

## polynomial regression
model.quad <- lm(height ~ flower + I(flower^2), data = data)
summary(model.quad)

# plot(data$height ~ data$flower)
pred <- data.frame(flower = 50:160)
pc <- predict(model.quad, int = "c", newdata = pred)
plot(data$height ~ data$flower)
matlines(pred$flower, pc, lty = c(1, 2, 2), col = "red")

# compare predicted and observed values
lim <- range(c(data$height, fitted(model), fitted(model.quad)))
plot(data$height, fitted(model), 
		xlab = "Observed", ylab = "Expected",
	 	xlim = lim, ylim = lim)
points(data$height, fitted(model.quad), col = "red")
abline(0, 1)

# compare error variance between two models
anova(model, model.quad)

# extend polynormial regression model to a higher dimensional one...
model.cube <- lm(height ~ flower + I(flower^2) + I(flower^3), data = data)
summary(model.cube)

# compare error variance between two models
anova(model.quad, model.cube)

## multi-linear regression with genetic background 
model.wgb <- lm(height ~ PC1 + PC2 + PC3 + PC4, data = data)
summary(model.wgb)
anova(model.wgb)

## multi-linear regression with all information
model.all <- lm(height ~ flower + I(flower^2) + PC1 + PC2 + PC3 + PC4, data = data)
summary(model.all)

# compare error variance
anova(model.all, model.wgb)

# compare between the simplest and final models
lim <- range(data$height, fitted(model), fitted(model.all))
plot(data$height, fitted(model), xlab = "Observed", ylab = "Fitted", xlim = lim, ylim = lim)
points(data$height, fitted(model.all), col = "red")
abline(0,1)


## simulations of a yield experiment with randomized complete block design
# set a seed for random number generation
set.seed(123)

# simulate the condition of an experimental field with 4 x 4 plots
# The blocks have unequal fertility among them
field.cond <- matrix(rep(c(4,2,-2,-4), each = 4), nrow = 4)
field.cond

# set block to consider the heterogeneity of field condition
block <- c("I", "II", "III", "IV")
blomat <- matrix(rep(block, each = 4), nrow = 4)
blomat

# assume that there are four varieties
variety <- c("A", "B", "C", "D")

# sample the order of the four varieties randomly
sample(variety)
sample(variety)

# allocate the varieties randomly to each column of the field
varmat <- matrix(c(sample(variety), sample(variety), 
			sample(variety), sample(variety)), nrow = 4)
varmat 

# simulate genetic ability of the varieties
g.value <- matrix(NA, 4, 4)
g.value[varmat == "A"] <- 4
g.value[varmat == "B"] <- 2
g.value[varmat == "C"] <- -2
g.value[varmat == "D"] <- -4
g.value

# simulate error variance (variation due to the heterogeneity of local environment)
e.value <- matrix(rnorm(16, sd = 2.5), 4, 4)
e.value

# simulate phenotypic values
grand.mean <- 50
simyield <- grand.mean + field.cond + g.value + e.value
simyield

# look at the simulated yield
op <- par(mfrow = c(2, 2))
image(t(field.cond))
for(i in 1:4) text((i-1) / 3, 0:3 / 3, blomat[,i])
image(t(g.value))
for(i in 1:4) text((i-1) / 3, 0:3 / 3, varmat[,i])
image(t(e.value))
image(t(simyield))
for(i in 1:4) text((i-1) / 3, 0:3 / 3, paste(varmat[,i], blomat[,i]))
par(op)

# unfold a matrix to a vector
as.vector(simyield)
as.vector(varmat)
as.vector(blomat)

# create a dataframe for the analysis of variance
simdata <- data.frame(variety = as.vector(varmat), block = as.vector(blomat), yield = as.vector(simyield))
simdata

# draw interaction plot
interaction.plot(simdata$block, simdata$variety, simdata$yield)

# perform the analysis of variance (ANOVA) with simulated data
res <- aov(yield ~ block + variety, data = simdata)
summary(res)

# perform ANOVA with a linear model
res <- lm(yield ~ block + variety, data = simdata)
anova(res)

# look inside the model
model.matrix(res)
summary(res)

## conduct calculation for ANOVA by ourselves
simdata <- simdata[order(simdata$block, simdata$variety),]
simdata
xij <- matrix(simdata$yield, nrow = 4)
xij
x.. <- mean(xij)
xi. <- apply(xij, 1, mean)
x.j <- apply(xij, 2, mean)

m <- nrow(xij)
r <- ncol(xij)
ss.blo <- sum((x.j - x..)^2) * m
ss.blo
ss.var <- sum((xi. - x..)^2) * r
ss.var
ss.err <- sum((sweep(sweep(xij, 1, xi.), 2, x.j) + x..)^2)
ss.err

ms.blo <- ss.blo / (r - 1)
ms.blo
ms.var <- ss.var / (m - 1)
ms.var
ms.err <- ss.err / ((m - 1) * (r - 1))
ms.err

f.value <- ms.var / ms.err
f.value

qf(1 - c(0.05, 0.01, 0.001), m - 1, (m - 1) * (r - 1))

p.value <- 1 - pf(f.value, m - 1, (m - 1) * (r - 1))
p.value


## simulations of a yield experiment with completely randomized design (CRD)
# completely randomized the plots of each variety in a field
varmat.crd <- matrix(sample(varmat), nrow = 4)
varmat.crd

# simulate genetic ability of the varieties
g.value.crd <- matrix(NA, 4, 4)
g.value.crd[varmat.crd == "A"] <- 4
g.value.crd[varmat.crd == "B"] <- 2
g.value.crd[varmat.crd == "C"] <- -2
g.value.crd[varmat.crd == "D"] <- -4
g.value.crd

# simulate phenotypic values
simyield.crd <- grand.mean + g.value.crd + field.cond + e.value
simyield.crd

# create a dataframe for the analysis of variance
simdata.crd <- data.frame(variety = as.vector(varmat.crd), 
							yield = as.vector(simyield.crd))
simdata.crd

# perform ANOVA
res <- lm(yield ~ variety, data = simdata.crd)
anova(res)
summary(res)

# perform multiple simulations
n.rep <- 100
p.rbd <- rep(NA, n.rep)
p.crd <- rep(NA, n.rep)
for(i in 1:n.rep) {
	# experiment with randomized block design
	varmat <- matrix(c(sample(variety), sample(variety), 
			sample(variety), sample(variety)), nrow = 4)
	g.value <- matrix(NA, 4, 4)
	g.value[varmat == "A"] <- 4
	g.value[varmat == "B"] <- 2
	g.value[varmat == "C"] <- -2
	g.value[varmat == "D"] <- -4
	e.value <- matrix(rnorm(16, sd = 2.5), 4, 4)
	simyield <- grand.mean + field.cond + g.value + e.value
	simdata <- data.frame(variety = as.vector(varmat), 
			block = as.vector(blomat), yield = as.vector(simyield))
	res <- lm(yield ~ block + variety, data = simdata)
	p.rbd[i] <- anova(res)$Pr[2]
	
	# experiment with completed randomized design
	varmat.crd <- matrix(sample(varmat), nrow = 4)
	g.value.crd <- matrix(NA, 4, 4)
	g.value.crd[varmat.crd == "A"] <- 4
	g.value.crd[varmat.crd == "B"] <- 2
	g.value.crd[varmat.crd == "C"] <- -2
	g.value.crd[varmat.crd == "D"] <- -4
	simyield.crd <- grand.mean + g.value.crd + field.cond + e.value
	simdata.crd <- data.frame(variety = as.vector(varmat.crd), 
							yield = as.vector(simyield.crd))
	res <- lm(yield ~ variety, data = simdata.crd)
	p.crd[i] <- anova(res)$Pr[1]
}
sum(p.rbd < 0.05) / n.rep
sum(p.crd < 0.05) / n.rep
sum(p.rbd < 0.01) / n.rep
sum(p.crd < 0.01) / n.rep



